from abc import abstractmethod
from itertools import chain

from ..tree.tree import get_data_property, TreeValue
from ...utils import SingletonMeta


class _BaseProcessor(metaclass=SingletonMeta):
    def _get_key_entries(*args, **kwargs):
        return [
            (index, tuple(sorted(get_data_property(value).keys())))
            for index, value in chain(enumerate(args), kwargs.items())
            if isinstance(value, TreeValue)
        ]

    @abstractmethod
    def get_key_set(self, *args, **kwargs):
        raise NotImplementedError  # pragma: no cover

    def check_arguments(self, mode, return_type, inherit, allow_missing, missing_func, subside, rise):
        if return_type is not None:
            if not isinstance(return_type, type):
                raise TypeError("Return type should be a type or none, but {type} found.".format(
                    type=repr(type(return_type).__name__)))
            elif not issubclass(return_type, TreeValue):
                raise TypeError("Tree value should be subclass of TreeValue, but {type} found.".format(
                    type=repr(return_type.__name__)
                ))



from functools import wraps
from typing import Union, Any, Mapping

from ..common import BaseTree, Tree
from ...utils import init_magic, build_tree

_DATA_PROPERTY = '_property__data'
_PRESERVED_PROPERTIES = {
    _DATA_PROPERTY,
}


def get_data_property(t: 'TreeValue') -> BaseTree:
    return getattr(t, _DATA_PROPERTY)


def _dict_unpack(t: Union['TreeValue', Mapping[str, Any]]) -> Union[BaseTree, Any]:
    if isinstance(t, BaseTree):
        return t
    elif isinstance(t, TreeValue):
        return get_data_property(t)
    elif isinstance(t, dict):
        return Tree({str(key): _dict_unpack(value) for key, value in t.items()})
    else:
        return t


def _init_decorate(init_func):
    @wraps(init_func)
    def _new_init_func(data):
        if isinstance(data, (TreeValue, dict)):
            _new_init_func(_dict_unpack(data))
        elif isinstance(data, BaseTree):
            init_func(data)
        else:
            raise TypeError(
                "Unknown initialization type for tree value - {type}.".format(type=repr(type(data).__name__)))

    return _new_init_func


@init_magic(_init_decorate)
class TreeValue:
    """
    Overview:
        Base framework of tree value. \
        And if the fast functions and operators are what you need, \
        please use `FastTreeValue` in `treevalue.tree.general`. \
        The `TreeValue` class is a light-weight framework just for DIY.
    """

    def __init__(self, data: Union[BaseTree, 'TreeValue', dict]):
        """
        Overview:
            Constructor of `TreeValue`.

        Arguments:
            - data: (:obj:`Union[BaseTree, 'TreeValue', dict]`): Original data to init a tree value, \
                can be a `BaseTree`, `TreeValue` or dict.

        Example:
            >>> TreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
            >>> # this is the tree:
            >>> # <root> -+--> a (1)
            >>> #         +--> b (2)
            >>> #         +--> x
            >>> #              +--> c (3)
            >>> #              +--> d (4)
        """
        setattr(self, _DATA_PROPERTY, data)

    @classmethod
    def __raw_value_to_value(cls, value):
        return cls(value) if isinstance(value, BaseTree) else value

    def __getattr__(self, key):
        """
        Overview:
            Get item from this tree value.

        Arguments:
            - key (:obj:`str`): Attribute name.

        Returns:
            - attr (:obj:): Target attribute value.

        Example:
            >>> t = TreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
            >>> t.a    # 1
            >>> t.b    # 2
            >>> t.x.c  # 3
        """
        _tree = get_data_property(self)
        if key in _tree.keys():
            value = get_data_property(self).__getitem__(key)
            return self.__raw_value_to_value(value)
        else:
            return self._attr_extern(key)

    def __setattr__(self, key, value):
        """
        Overview:
            Set sub node to this tree value.

        Arguments:
            - key (:obj:`str`): Attribute name.
            - value (:obj:): Sub value.

        Example:
            >>> t = TreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
            >>> t.a = 3                 # t will be TreeValue({'a': 3, 'b': 2, 'x': {'c': 3, 'd': 4}})
            >>> t.b = {'x': 1, 'y': 2}  # t will be TreeValue({'a': 3, 'b': {'x': 1, 'y': 2}, 'x': {'c': 3, 'd': 4}})
        """
        if key in _PRESERVED_PROPERTIES:
            object.__setattr__(self, key, value)
        else:
            if isinstance(value, TreeValue):
                value = get_data_property(value)
            return get_data_property(self).__setitem__(key, value)

    def __delattr__(self, key):
        """
        Overview:
            Delete attribute from tree value.

        Arguments:
            - key (:obj:`str`): Attribute name.

        Example:
            >>> t = TreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
            >>> del t.a    # t will be TreeValue({'b': 2, 'x': {'c': 3, 'd': 4}})
            >>> del t.x.c  # t will be TreeValue({'b': 2, 'x': {'d': 4}})
        """
        if key in _PRESERVED_PROPERTIES:
            raise AttributeError("Unable to delete attribute {attr}.".format(attr=repr(key)))
        else:
            return get_data_property(self).__delitem__(key)

    def __contains__(self, key) -> bool:
        """
        Overview:
            Check if attribute is in this tree value.

        Arguments:
            - key (:obj:`str`): Attribute name.

        Returns:
            - exist (:obj:`bool`): If attribute is in this tree value.

        Example:
            >>> t = TreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
            >>> 'a' in t  # True
            >>> 'b' in t  # True
            >>> 'c' in t  # False
        """
        return key in get_data_property(self).keys()

    def __iter__(self):
        """
        Overview:
            Get iterator of this tree value.

        Returns:
            - iter (:obj:`iter`): Iterator for keys and values.

        Example:
            >>> t = TreeValue({'a': 1, 'b': 2, 'x': 3})
            >>> for key, value in t:
            >>>     print(key, value)

            The output will be:

            >>> a 1
            >>> b 2
            >>> x 3
        """
        for key, value in get_data_property(self).items():
            yield key, self.__raw_value_to_value(value)

    def __len__(self):
        """
        Overview:
            Get count of the keys.

        Returns:
            - length (:obj:`int`): Count of the keys.

        Example:
            >>> t = TreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
            >>> len(t)    # 3
            >>> len(t.x)  # 2
        """
        return len(get_data_property(self))

    def __bool__(self) -> bool:
        """
        Overview:
            Check if the tree value is not empty.

        Returns:
            - non_empty (:obj:`bool`): Not empty or do.

        Example:
            >>> t = TreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}, 'e': {}})
            >>> not not t    # True
            >>> not not t.x  # True
            >>> not not t.e  # False
        """
        return not not get_data_property(self)

    def __repr__(self):
        """
        Overview:
            Get representation format of tree value.

        Returns:
            - repr (:obj:`str`): Representation string.

        Example:
            >>> t = TreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
            >>> repr(t)  # <TreeValue 0xffffffff keys: ['a', 'b', 'x']>, the is may be different
        """
        _tree = get_data_property(self)
        return "<{cls} {id} keys: {keys}>".format(
            cls=self.__class__.__name__,
            id=hex(id(_tree.actual())),
            keys=repr(sorted(_tree.keys()))
        )

    def __str__(self):
        """
        Overview:
            Get the structure of the tree.

        Returns:
            - str (:obj:`str`): Returned string.
        """
        return str(build_tree(
            self,
            represent=lambda x: repr(x),
            iterate=lambda x: iter(x),
            recurse=lambda x: isinstance(x, TreeValue),
        ))

    def __hash__(self):
        """
        Overview:
            Hash value of current object.

        Returns:
            - hash (:obj:`int`): Hash code of current object.
        """
        return hash(get_data_property(self))

    def __eq__(self, other):
        """
        Overview:
            Check the equality of two tree values.

        Arguments:
            - other (:obj:`TreeValue`): Another tree value.

        Returns:
            - equal (:obj:`bool`): Equality.

        Example:
            >>> t = TreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
            >>> clone(t) == t                                                # True
            >>> t == TreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 5}})      # False
            >>> t == TreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})      # True
            >>> t == FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})  # False (type not match)
        """
        if self is other:
            return True
        elif type(other) == self.__class__:
            return get_data_property(self) == get_data_property(other)
        else:
            return False

    def _attr_extern(self, key):
        """
        Overview:
            External protected function for support the unfounded attributes. \
            Default is raise a `KeyError`.

        Arguments:
            - key (:obj:`str`): Attribute name.

        Returns:
            - return (:obj:): Anything you like, \
                and if it is not able to validly return anything, \
                just raise an exception here.
        """
        raise KeyError("Attribute {key} not found.".format(key=repr(key)))



from abc import ABCMeta, abstractmethod
from typing import List, Union, Callable, Any

from treevalue.utils import build_tree


class BaseTree(metaclass=ABCMeta):
    """
    Overview:
        Base abstract structure of a tree data class.
    """

    @abstractmethod
    def __getitem__(self, key):
        """
        Overview:
            Get item from this tree node, raw value and tree node returns.

        Arguments:
            - key (:obj:`str`): Key of the sub tree node.

        Returns:
            - return (:obj:`Any`): Tree node or raw value (if the sub node is value node).

        Example:
            >>> t = Tree({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
            >>> t['a']       # 1
            >>> t['x']       # Tree({'c': 3, 'd': 4})
            >>> t['x']['c']  # 3
        """

        raise NotImplementedError  # pragma: no cover

    @abstractmethod
    def __setitem__(self, key, value):
        """
        Overview:
            Set item to this tree node, raw value and tree node supported.

        Arguments:
            - key (:obj:`str`): Key of the sub tree node.
            - value (:obj:`Any`): Given mode or value to be set.

        Example:
            >>> t = Tree({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
            >>> t['a'] = 3
            >>> t  # Tree({'a': 3, 'b': 2, 'x': {'c': 3, 'd': 4}})
            >>> t['x']['e'] = 10
            >>> t  # Tree({'a': 3, 'b': 2, 'x': {'c': 3, 'd': 4, 'e': 10}})
        """
        raise NotImplementedError  # pragma: no cover

    @abstractmethod
    def __delitem__(self, key):
        """
        Overview:
            Delete item from this tree node.

        Arguments:
            - key (:obj:`str`): Key of the sub tree node.

        Example:
            >>> t = Tree({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
            >>> del t['a']
            >>> t  # Tree({'b': 2, 'x': {'c': 3, 'd': 4}})
            >>> del t['x']['c']
            >>> t  # Tree({'b': 2, 'x': {'d': 4}})
        """
        raise NotImplementedError  # pragma: no cover

    def json(self):
        """
        Overview:
            Get full json data of this tree.

        Returns:
            - json (:obj:`dict`): Json data

        Example:
            >>> t = Tree({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
            >>> t.json()  # {'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}}
        """
        return {
            key: value.json() if isinstance(value, BaseTree) else value
            for key, value in self.items()
        }

    @abstractmethod
    def view(self, path: List[str]):
        """
        Overview:
            Get view of the current tree.

        Arguments:
            - path (:obj:`List[str]`): Viewing path.

        Returns:
            - viewed_tree (:obj:`TreeView`): Viewed tree.

        Example:
            >>> t = Tree({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
            >>> t1 = t['x']
            >>> t2 = t.view(["x"])
            >>>
            >>> t1['c']  # 3
            >>> t2['c']  # 3
            >>>
            >>> t.x = Tree({'c': 5, 'd': 6})
            >>> t1['c']  # 3
            >>> t2['c']  # 5
        """

        raise NotImplementedError  # pragma: no cover

    @abstractmethod
    def clone(self, copy_value: Union[None, bool, Callable, Any] = None):
        """
        Overview:
            Fully clone this tree.

        Arguments:
            - copy_value (:obj:`Union[None, bool, Callable, Any]`): Deep copy value or not, \
                default is `None` which means do not deep copy the values. \
                If deep copy is required, just set it to `True`.

        Returns:
            - cloned (:obj:`Tree`): Cloned new tree.

        Example:
            >>> t = Tree({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
            >>> t2 = t.clone()
            >>> t2                 # Tree({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
            >>> t2 is t            # False
            >>> t2['x'] is t['x']  # False
        """
        raise NotImplementedError  # pragma: no cover

    @abstractmethod
    def items(self):
        """
        Overview:
            Get item iterator of this tree node.

        Returns:
            - iter (:obj:`iter`): Item iterator of this tree node.

        Example:
            >>> t = Tree({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
            >>> for key, value in t.items():
            >>>     print(key, value)

            The output should be

            >>> a 1
            >>> b 2
            >>> x <Tree 0x7f74f6daaf60 keys: ['c', 'd']>
        """
        raise NotImplementedError  # pragma: no cover

    @abstractmethod
    def keys(self):
        """
        Overview:
            Get key iterator of this tree node.

        Returns:
            - iter (:obj:`iter`): Key iterator of this tree node.

        Example:
            >>> t = Tree({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
            >>> for key in t.keys():
            >>>     print(key)

            The output should be

            >>> a
            >>> b
            >>> x
        """
        raise NotImplementedError  # pragma: no cover

    @abstractmethod
    def values(self):
        """
        Overview:
            Get value iterator of this tree node.

        Returns:
            - iter (:obj:`iter`): Value iterator of this tree node.

        Example:
            >>> t = Tree({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
            >>> for value in t.values():
            >>>     print(value)

            The output should be

            >>> 1
            >>> 2
            >>> <Tree 0x7f74f6daaf60 keys: ['c', 'd']>
        """
        raise NotImplementedError  # pragma: no cover

    @abstractmethod
    def actual(self):
        """
        Overview:
            Get actual tree node of current tree object.
            If this is a `Tree` object, just return itself.
            If this is a `TreeView` object, it will return the actually viewed tree node.

        Returns:
            - actual (:obj:`Tree`): Actual tree node.
        """
        raise NotImplementedError  # pragma: no cover

    def __len__(self):
        return len(self.keys())

    def __bool__(self):
        return len(self.keys()) > 0

    def __hash__(self):
        return hash(tuple([(key, value) for key, value in sorted(self.items())]))

    def __eq__(self, other):
        if other is self:
            return True
        elif isinstance(other, BaseTree):
            if set(self.keys()) == set(other.keys()):
                for key in self.keys():
                    if self[key] != other[key]:
                        return False
                return True
            else:
                return False
        else:
            return False

    def __repr__(self):
        return '<{cls} {id} keys: {keys}>'.format(
            cls=self.__class__.__name__,
            id=hex(id(self.actual())),
            keys=repr(sorted(self.keys()))
        )

    def __str__(self):
        """
        Overview:
            Return tree-formatted string.

        Returns:
            - string (:obj:`str`): Tree-formatted string.

        Example:
            >>> t = Tree({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}, 'z': [1, 2], 'v': raw({'1': '2'})})
            >>> print(t)

            The output will be

            >>> <Tree 0x7f9fa48b9588 keys: ['a', 'b', 'v', 'x', 'z']>
            >>> ├── 'a' --> 1
            >>> ├── 'b' --> 2
            >>> ├── 'v' --> {'1': '2'}
            >>> ├── 'x' --> <Tree 0x7f9fa48b95c0 keys: ['c', 'd']>
            >>> │   ├── 'c' --> 3
            >>> │   └── 'd' --> 4
            >>> └── 'z' --> [1, 2]
        """
        return str(build_tree(
            self,
            represent=lambda value: repr(value),
            iterate=lambda value: value.items(),
            recurse=lambda value: isinstance(value, BaseTree),
        ))



import warnings

from .base import _BaseProcessor


class _StrictProcessor(_BaseProcessor):
    def get_key_set(self, *args, **kwargs):
        key_entries = self._get_key_entries(*args, **kwargs)
        first_index, first_keyset = key_entries[0]
        for index, keyset in key_entries[1:]:
            if keyset != first_keyset:
                raise KeyError(
                    "Argument keys not match in strict mode, key set of argument {a1} is {ks1} but {a2} in {ks2}.".format(
                        a1=repr(first_index), ks1=repr(first_keyset),
                        a2=repr(index), ks2=repr(keyset),
                    ))

        return first_keyset

    def check_arguments(self, mode, return_type, inherit,
                        allow_missing, missing_func, subside, rise):
        _BaseProcessor.check_arguments(self, mode, return_type, inherit,
                                       allow_missing, missing_func, subside, rise)
        if allow_missing:
            warnings.warn(RuntimeWarning("Allow missing detected, but cannot applied in strict mode."))



from functools import wraps
from inspect import signature, Parameter
from itertools import chain
from typing import Callable


def args_iter(*args, **kwargs):
    """
    Overview:
        Iterate all the arguments with index and value.
        If argument is in `args`, the index should be integer increasing from 0.
        If argument is in `kwargs`, the index should be string which meaning the argument's name.
        The numeric indices will appear before the string indices,
        and **the order of the string indices are not approved**.

    Arguments:
        - args (:obj:`Tuple[Any]`): Argument list
        - kwargs (:obj:`Dict[str, Any]`): Argument mapping

    Example:
        >>> for index, value in args_iter(1, 2, 3, a=1, b=2, c=3)):
        >>>     print(index, value)

        The output should be

        >>> 0 1
        >>> 1 2
        >>> 2 3
        >>> a 1
        >>> b 2
        >>> c 3
    """
    for _index, _item in chain(enumerate(args), sorted(kwargs.items())):
        yield _index, _item


def dynamic_call(func: Callable):
    """
    Overview:
        Decorate function to dynamic-call-supported function.

    Arguments:
        - func (:obj:`Callable`): Original function to be decorated.

    Returns:
        - new_func (:obj:`Callable`): Decorated function.

    Example:
        >>> dynamic_call(lambda x, y: x ** y)(2, 3)  # 8
        >>> dynamic_call(lambda x, y: x ** y)(2, 3, 4)  # 8, 3rd is ignored
        >>> dynamic_call(lambda x, y, t, *args: (args, (t, x, y)))(1, 2, 3, 4, 5)  # ((4, 5), (3, 1, 2))
        >>> dynamic_call(lambda x, y: (x, y))(y=2, x=1)  # (1, 2), key word supported
        >>> dynamic_call(lambda x, y, **kwargs: (kwargs, x, y))(1, k=2, y=3)  # ({'k': 2}, 1, 3)
    """
    enable_args, args_count = False, 0
    enable_kwargs, kwargs_set = False, set()

    for name, param in signature(func, follow_wrapped=False).parameters.items():
        if param.kind in {Parameter.POSITIONAL_ONLY, Parameter.POSITIONAL_OR_KEYWORD}:
            args_count += 1
        if param.kind in (Parameter.KEYWORD_ONLY, Parameter.POSITIONAL_OR_KEYWORD):
            kwargs_set |= {name}
        if param.kind == Parameter.VAR_POSITIONAL:
            enable_args = True
        if param.kind == Parameter.VAR_KEYWORD:
            enable_kwargs = True

    def _get_args(*args):
        return args if enable_args else args[:args_count]

    def _get_kwargs(**kwargs):
        return kwargs if enable_kwargs else {key: value for key, value in kwargs.items() if key in kwargs_set}

    @wraps(func)
    def _new_func(*args, **kwargs):
        return func(*_get_args(*args), **_get_kwargs(**kwargs))

    return _new_func



from .base import _BaseProcessor


class _LeftProcessor(_BaseProcessor):
    def get_key_set(self, *args, **kwargs):
        key_entries = sorted(
            self._get_key_entries(*args, **kwargs),
            key=lambda x: (0, x[0]) if isinstance(x[0], int) else (1, x[0])
        )

        _, first_keyset = key_entries[0]
        return first_keyset



from functools import reduce

from .base import _BaseProcessor


class _InnerProcessor(_BaseProcessor):
    def get_key_set(self, *args, **kwargs):
        return reduce(lambda x, y: x & y, [set(keyset) for _, keyset in self._get_key_entries(*args, **kwargs)])



from enum import IntEnum, unique
from functools import wraps, partial
from typing import Type, TypeVar, Optional, Mapping, Union, Callable, Any

import enum_tools

from .inner import _InnerProcessor
from .left import _LeftProcessor
from .outer import _OuterProcessor
from .strict import _StrictProcessor
from ..common import raw
from ..tree.tree import TreeValue
from ..tree.utils import rise as rise_func
from ..tree.utils import subside as subside_func
from ...utils import int_enum_loads, SingletonMark


@enum_tools.documentation.document_enum
@int_enum_loads(name_preprocess=str.upper)
@unique
class TreeMode(IntEnum):
    """
    Overview:
        Four mode of the tree calculation
    """
    STRICT = 1  # doc: Strict mode, which means the keys should be one to one in every trees.
    LEFT = 2  # doc: Left mode, the keys of the result is relied on the left value.
    INNER = 3  # doc: Inner mode, the keys of the result is relied on the intersection of the trees' key set.
    OUTER = 4  # doc: Outer mode, the keys of the result is relied on the union of the trees' key set.


_MODE_PROCESSORS = {
    TreeMode.STRICT: _StrictProcessor(),
    TreeMode.LEFT: _LeftProcessor(),
    TreeMode.INNER: _InnerProcessor(),
    TreeMode.OUTER: _OuterProcessor(),
}


def _any_getattr(value):
    class _AnyClass:
        def __getattr__(self, item):
            return value

    return _AnyClass()


_ClassType = TypeVar("_ClassType", bound=TreeValue)
MISSING_NOT_ALLOW = SingletonMark("missing_not_allow")


def func_treelize(mode: Union[str, TreeMode] = 'strict',
                  return_type: Optional[Type[_ClassType]] = TreeValue, inherit: bool = True,
                  missing: Union[Any, Callable] = MISSING_NOT_ALLOW,
                  subside: Union[Mapping, bool, None] = None,
                  rise: Union[Mapping, bool, None] = None):
    """
    Overview:
        Wrap a common function to tree-supported function.

    Arguments:
        - mode (:obj:`Union[str, TreeMode]`): Mode of the wrapping (string or TreeMode both okay), default is `strict`.
        - return_type (:obj:`Optional[Type[_ClassType]]`): Return type of the wrapped function, default is `TreeValue`.
        - inherit (:obj:`bool`): Allow inherit in wrapped function, default is `True`.
        - missing (:obj:`Union[Any, Callable]`): Missing value or lambda generator of when missing, \
            default is `MISSING_NOT_ALLOW`, which means raise `KeyError` when missing detected.
        - subside (:obj:`Union[Mapping, bool, None]`): Subside enabled to function's arguments or not, \
            and subside configuration, default is `None` which means do not use subside. \
            When subside is `True`, it will use all the default arguments in `subside` function.
        - rise (:obj:`Union[Mapping, bool, None]`): Rise enabled to function's return value or not, \
            and rise configuration, default is `None` which means do not use rise. \
            When rise is `True`, it will use all the default arguments in `rise` function. \
            (Not recommend to use auto mode when your return structure is not so strict.)

    Returns:
        - decorator (:obj:`Callable`): Wrapper for tree-supported function.

    Example:
        >>> @func_treelize()
        >>> def ssum(a, b):
        >>>     return a + b  # the a and b will be integers, not TreeValue
        >>>
        >>> t1 = TreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
        >>> t2 = TreeValue({'a': 11, 'b': 22, 'x': {'c': 33, 'd': 5}})
        >>> ssum(1, 2)    # 3
        >>> ssum(t1, t2)  # TreeValue({'a': 12, 'b': 24, 'x': {'c': 36, 'd': 9}})
    """
    mode = TreeMode.loads(mode)
    if missing is MISSING_NOT_ALLOW:
        allow_missing = False
        missing_func = None
    else:
        allow_missing = True
        missing_func = missing if hasattr(missing, '__call__') else (lambda: missing)

    if subside is not None and not isinstance(subside, dict):
        subside = {} if subside else None
    if rise is not None and not isinstance(rise, dict):
        rise = {} if rise else None

    _MODE_PROCESSORS[mode].check_arguments(mode, return_type, inherit,
                                           allow_missing, missing_func,
                                           subside, rise)

    def _value_wrap(item, index):
        if isinstance(item, TreeValue):
            def _get_from_key(key):
                if key in item:
                    return item.__getattr__(key)
                elif allow_missing:
                    return missing_func()
                else:
                    raise KeyError("Missing is off, key {key} not found in {item}.".format(
                        key=repr(key), item=repr(item),
                    ))

            return _get_from_key
        elif inherit:
            return _any_getattr(item).__getattr__
        else:
            raise TypeError("Inherit is off, tree value expected but {type} found in args {index}.".format(
                type=repr(type(item).__name__), index=repr(index),
            ))

    def _decorator(func):
        def _recursion(*args, **kwargs) -> Optional[_ClassType]:
            if all([not isinstance(item, TreeValue) for item in args]) \
                    and all([not isinstance(value, TreeValue) for value in kwargs.values()]):
                return func(*args, **kwargs)

            pargs = [_value_wrap(item, index) for index, item in enumerate(args)]
            pkwargs = {key: _value_wrap(value, key) for key, value in kwargs.items()}

            _data = {
                key: raw(_recursion(
                    *(item(key) for item in pargs),
                    **{key_: value(key) for key_, value in pkwargs.items()}
                )) for key in sorted(_MODE_PROCESSORS[mode].get_key_set(*args, **kwargs))
            }
            return return_type(_data) if return_type is not None else None

        @wraps(func)
        def _new_func(*args, **kwargs):
            if subside is not None:
                args = [subside_func(item, **subside) for item in args]
                kwargs = {key: subside_func(value, **subside) for key, value in kwargs.items()}

            _result = _recursion(*args, **kwargs)
            if rise is not None:
                _result = rise_func(_result, **rise)

            return _result

        return _new_func

    return _decorator


_CONFIGS_TAG = '__configs__'


def _tree_check(clazz, allow_none=False):
    if clazz is None:
        if not allow_none:
            raise TypeError("Tree value class is not allowed to be none, but None found.")
        else:
            return

    if not isinstance(clazz, type):
        raise TypeError("Tree value class should be a type, but {type} found.".format(
            type=repr(type(clazz).__name__)
        ))
    elif not issubclass(clazz, TreeValue):
        raise TypeError("Tree value class should be subclass of TreeValue, but {type} found.".format(
            type=repr(clazz.__name__)
        ))


def _class_config(return_type: Optional[Type[_ClassType]] = None,
                  allow_none_return_type: bool = True,
                  clazz_must_be_tree_value: bool = True):
    _tree_check(return_type, allow_none=allow_none_return_type)

    def _decorator(clazz: type) -> type:
        if clazz_must_be_tree_value:
            _tree_check(clazz, allow_none=False)

        config = _get_configs(clazz)
        config.update(dict(
            return_type=return_type,
        ))
        setattr(clazz, _CONFIGS_TAG, config)
        return clazz

    return _decorator


def tree_class(return_type: Optional[Type[_ClassType]] = None):
    """
    Overview:
        Wrap tree configs for ``TreeValue`` class.

    Arguments:
        - return_type (:obj:`Optional[Type[_ClassType]]`): Default return type of current class, \
            default is ``None`` which means use the class itself.

    Returns:
        - decorator (:obj:`Callable`): A class decorator.
    """
    return _class_config(
        return_type, True, True,
    )


def _get_configs(clazz: type):
    return dict(getattr(clazz, _CONFIGS_TAG, None) or {})


AUTO_DETECT_RETURN_TYPE = SingletonMark("auto_detect_return_type")


def _auto_detect_type(clazz: type):
    return _get_configs(clazz).get('return_type', None) or clazz


def method_treelize(mode: Union[str, TreeMode] = 'strict',
                    return_type: Optional[Type[_ClassType]] = AUTO_DETECT_RETURN_TYPE, inherit: bool = True,
                    missing: Union[Any, Callable] = MISSING_NOT_ALLOW,
                    subside: Union[Mapping, bool, None] = None,
                    rise: Union[Mapping, bool, None] = None):
    """
    Overview:
        Wrap a common instance method to tree-supported method.

    Attention:
        - This decorator can only used to instance method, usage with class method may cause unconditional fatal.
        - When decorated instance method is called, the `self` argument will be no longer the class instance, \
            but the single element of the tree instead.

    Arguments:
        - mode (:obj:`Union[str, TreeMode]`): Mode of the wrapping (string or TreeMode both okay), default is `strict`.
        - return_type (:obj:`Optional[Type[_ClassType]]`): Return type of the wrapped function, \
            default is `AUTO_DETECT_RETURN_VALUE`, which means automatically use the decorated method's class.
        - inherit (:obj:`bool`): Allow inherit in wrapped function, default is `True`.
        - missing (:obj:`Union[Any, Callable]`): Missing value or lambda generator of when missing, \
            default is `MISSING_NOT_ALLOW`, which means raise `KeyError` when missing detected.
        - subside (:obj:`Union[Mapping, bool, None]`): Subside enabled to function's arguments or not, \
            and subside configuration, default is `None` which means do not use subside. \
            When subside is `True`, it will use all the default arguments in `subside` function.
        - rise (:obj:`Union[Mapping, bool, None]`): Rise enabled to function's return value or not, \
            and rise configuration, default is `None` which means do not use rise. \
            When rise is `True`, it will use all the default arguments in `rise` function. \
            (Not recommend to use auto mode when your return structure is not so strict.)

    Returns:
        - decorator (:obj:`Callable`): Wrapper for tree-supported method.

    Example:
        >>> class MyTreeValue(TreeValue):
        >>>     @method_treelize()
        >>>     def append(self, *args):
        >>>         return sum([self, *args])  # the self will be the integers, not MyTreeValue
        >>>
        >>> t1 = MyTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
        >>> t2 = MyTreeValue({'a': 11, 'b': 22, 'x': {'c': 33, 'd': 5}})
        >>> t1.append(2)   # MyTreeValue({'a': 3, 'b': 4, 'x': {'c': 5, 'd': 6}})
        >>> t1.append(t2)  # MyTreeValue({'a': 12, 'b': 24, 'x': {'c': 36, 'd': 9}})
    """

    def _decorator(method):
        @wraps(method)
        def _new_method(self, *args, **kwargs):
            rt = _auto_detect_type(self.__class__) if return_type is AUTO_DETECT_RETURN_TYPE else return_type
            return func_treelize(mode, rt, inherit, missing, subside, rise)(method)(self, *args, **kwargs)

        return _new_method

    return _decorator


def utils_class(return_type: Type[_ClassType]):
    """
    Overview:
        Wrap tree configs for utils class.

    Arguments:
        - return_type (:obj:`Optional[Type[_ClassType]]`): Default return type of current class.

    Returns:
        - decorator (:obj:`Callable`): A class decorator.

    Examples:
        >>> class MyTreeValue(TreeValue):
        >>>     pass
        >>>
        >>> @utils_class(return_type=MyTreeValue)
        >>> class MyTreeUtils:
        >>>     @classmethod
        >>>     @classmethod_treelize()
        >>>     def add(cls, a, b):
        >>>         return a + b
        >>>
        >>> t1 = TreeValue({'a': 1, 'b': 2})
        >>> t2 = TreeValue({'a': 3, 'b': 4})
        >>> MyTreeUtils.add(t1, t2)  # MyTreeValue({'a': 4, 'b': 6})
    """
    return _class_config(
        return_type, False, False,
    )


def classmethod_treelize(mode: Union[str, TreeMode] = 'strict',
                         return_type: Optional[Type[_ClassType]] = AUTO_DETECT_RETURN_TYPE, inherit: bool = True,
                         missing: Union[Any, Callable] = MISSING_NOT_ALLOW,
                         subside: Union[Mapping, bool, None] = None,
                         rise: Union[Mapping, bool, None] = None):
    """
    Overview:
        Wrap a common class method to tree-supported method.

    Attention:
        - This decorator can only used to class method, usage with instance method may cause unconditional fatal.
        - When decorated instance method is called, the `cls` argument will still be the calling class.

    Arguments:
        - mode (:obj:`Union[str, TreeMode]`): Mode of the wrapping (string or TreeMode both okay), default is `strict`.
        - return_type (:obj:`Optional[Type[_ClassType]]`): Return type of the wrapped function, \
            default is `AUTO_DETECT_RETURN_VALUE`, which means automatically use the decorated method's class.
        - inherit (:obj:`bool`): Allow inherit in wrapped function, default is `True`.
        - missing (:obj:`Union[Any, Callable]`): Missing value or lambda generator of when missing, \
            default is `MISSING_NOT_ALLOW`, which means raise `KeyError` when missing detected.
        - subside (:obj:`Union[Mapping, bool, None]`): Subside enabled to function's arguments or not, \
            and subside configuration, default is `None` which means do not use subside. \
            When subside is `True`, it will use all the default arguments in `subside` function.
        - rise (:obj:`Union[Mapping, bool, None]`): Rise enabled to function's return value or not, \
            and rise configuration, default is `None` which means do not use rise. \
            When rise is `True`, it will use all the default arguments in `rise` function. \
            (Not recommend to use auto mode when your return structure is not so strict.)

    Returns:
        - decorator (:obj:`Callable`): Wrapper for tree-supported class method.

    Example:
        >>> class TestUtils:
        >>>     @classmethod
        >>>     @classmethod_treelize(return_type=TreeValue)
        >>>     def add(cls, a, b):
        >>>         return cls, a + b
        >>>
        >>> TestUtils.add(
        >>>     TreeValue({'a': 1, 'b': 2}),
        >>>     TreeValue({'a': 11, 'b': 23}),
        >>> )  # TreeValue({'a': (TestUtils, 12), 'b': (TestUtils, 25)})
    """

    def _decorator(method):
        @wraps(method)
        def _new_method(cls, *args, **kwargs):
            rt = _auto_detect_type(cls) if return_type is AUTO_DETECT_RETURN_TYPE else return_type
            return func_treelize(mode, rt, inherit, missing, subside, rise)(partial(method, cls))(*args, **kwargs)

        return _new_method

    return _decorator



from copy import deepcopy
from functools import wraps
from typing import Dict, Any, Union, List, Callable

from .base import BaseTree
from ...utils import init_magic


class _RawWrapped:
    def __init__(self, value):
        self.__value = value

    @property
    def value(self):
        return self.__value


_RAW_NEEDED_TYPES = (dict,)


def raw(value) -> _RawWrapped:
    """
    Overview:
        Wrap raw value to init tree or set item, can be used for dict.
        Can only performs when value is a dict object, otherwise just return the original value.

    Arguments:
        - value (:obj:): Original value.

    Returns:
        - wrapped (:obj:`_RawWrapped`): Wrapped value.

    Example:
        >>> t = Tree({
        >>>     'a': raw({'a': 1, 'b': 2}),
        >>>     'b': raw({'a': 3, 'b': 4}),
        >>>     'x': {
        >>>         'c': raw({'a': 5, 'b': 6}),
        >>>         'd': raw({'a': 7, 'b': 8}),
        >>>     }
        >>> })
        >>>
        >>> t['a']  # {'a': 1, 'b': 2}
        >>> t['b']  # {'a': 3, 'b': 4}
        >>> t['x']['c']  # {'a': 5, 'b': 6}
        >>> t['x']['d']  # {'a': 7, 'b': 8}
        >>>
        >>> t['a'] = raw({'a': 9, 'b': 10})
        >>> t['a']  # {'a': 9, 'b': 10}
        >>> t['a'] = {'a': 9, 'b': 10}
        >>> t['a']  # should be a Tree object when raw not used
    """
    if not isinstance(value, _RawWrapped) and isinstance(value, _RAW_NEEDED_TYPES):
        return _RawWrapped(value)
    else:
        return value


def _unraw(value):
    if isinstance(value, _RawWrapped):
        return value.value
    else:
        return value


def _to_tree_decorator(init_func):
    @wraps(init_func)
    def _new_init_func(data):
        if isinstance(data, BaseTree):
            _new_init_func(_tree_dump(data))
        elif isinstance(data, dict):
            init_func({
                str(key): Tree(value) if isinstance(value, dict) else _unraw(value)
                for key, value in data.items()
            })
        else:
            raise TypeError(
                "Dict value expected for dispatch value but {type} actually.".format(type=repr(type(data).__name__)))

    return _new_init_func


def _copy_func(copy):
    if hasattr(copy, '__call__'):
        return copy
    elif copy is None or isinstance(copy, (bool,)):
        return (lambda x: deepcopy(x)) if copy else (lambda x: x)
    elif isinstance(copy, (list, tuple)):
        dumper, loader = copy[:2]
        return lambda x: loader(dumper(x))
    else:
        dumper, loader = getattr(copy, 'dumps'), getattr(copy, 'loads')
        return lambda x: loader(dumper(x))


def _tree_dump(tree: 'BaseTree', copy_value: Union[None, bool, Callable] = None):
    copy_value = _copy_func(copy_value)

    def _recursion(t):
        if isinstance(t, BaseTree):
            return {key: _recursion(value) for key, value in t.items()}
        else:
            return raw(copy_value(t))

    return _recursion(tree.actual())


@init_magic(_to_tree_decorator)
class Tree(BaseTree):
    """
    Overview:
        Tree node data model, based on `BaseTree`.
    """

    def __init__(self, data: Union[Dict[str, Union['Tree', Any]], 'Tree']):
        """
        Overview:
            Constructor of `Tree`, can be `dict`, `Tree`, `TreeView`.
            When dict passed in, a new tree structure will be created once.
            When `Tree` or `TreeView` passed in, a fully copy will be constructed in this object.

        Arguments:
            - data (:obj:`Union[Dict[str, Union['Tree', Any]], 'Tree']`): Any data can be parsed into `Tree`.

        Example:
            >>> t = Tree({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})  # t is a new tree structure
            >>> t2 = Tree(t)                                       # t2 is a full copy of t1
            >>> t3 = Tree({'a': t, 'b': t2})                       # t3 is a tree with subtree of t and t2 (not copy)
        """
        self.__dict = data

    def __check_key_exist(self, key):
        if key not in self.__dict.keys():
            raise KeyError("Key {key} not found.".format(key=repr(key)))

    def __getitem__(self, key):
        self.__check_key_exist(key)
        return self.__dict[key]

    def __setitem__(self, key, value):
        if isinstance(value, dict):
            value = Tree(value)
        self.__dict[key] = _unraw(value)

    def __delitem__(self, key):
        self.__check_key_exist(key)
        del self.__dict[key]

    def view(self, path: List[str]):
        from .view import TreeView
        return TreeView(self, path)

    def clone(self, copy_value: Union[None, bool, Callable, Any] = None):
        return self.__class__(_tree_dump(self, copy_value))

    def items(self):
        return self.__dict.items()

    def keys(self):
        return self.__dict.keys()

    def values(self):
        return self.__dict.values()

    def actual(self):
        return self



from enum import IntEnum, unique
from functools import lru_cache
from typing import Type, Optional, Callable, TypeVar, Any

_EnumType = TypeVar('_EnumType', bound=IntEnum)


def _default_value_preprocess(value: int):
    return value


def _default_name_preprocess(name: str):
    return name


def _get_default_external_preprocess(enum_class: Type[_EnumType]):
    def _default_external_preprocess(data):
        raise TypeError('Unknown type {type} for loads to {cls}.'.format(
            type=repr(type(data).__name__), cls=repr(enum_class.__name__),
        ))

    return _default_external_preprocess


def int_enum_loads(enable_int: bool = True, value_preprocess: Optional[Callable[[int, ], int]] = None,
                   enable_str: bool = True, name_preprocess: Optional[Callable[[str, ], str]] = None,
                   external_process: Optional[Callable[[Any, ], Optional[_EnumType]]] = None):
    """
    Overview:
        Decorate a int enum class with a new `loads` class method.

    Arguments:
        - enable_int (:obj:`bool`): Enable int parse, default is `True`.
        - value_preprocess (:obj:`Optional[Callable[[int, ], int]]`): Preprocessor of value, \
            default is `None` which means no change.
        - enable_str (:obj:`bool`): Enable str parse, default is `True`.
        - name_preprocess (:obj:`Optional[Callable[[str, ], str]]`): Preprocessor of name, \
            default is `None` which means no change.
        - external_process (:obj:`Optional[Callable[[Any, ], Optional[_EnumType]]]`): External processor \
            for the unprocessable data, default is `None` which means raise a `KeyError`.

    Examples:
        - Simple usage

        >>> from enum import IntEnum, unique
        >>>
        >>> @int_enum_loads()
        >>> @unique
        >>> class MyEnum(IntEnum):
        >>>     A = 1
        >>>     B = 2
        >>>
        >>> MyEnum.loads(1)    # MyEnum.A
        >>> MyEnum.loads('A')  # MyEnum.A
        >>> MyEnum.loads(2)    # MyEnum.B
        >>> MyEnum.loads('B')  # MyEnum.B
        >>> MyEnum.loads(-1)   # KeyError
        >>> MyEnum.loads('a')  # KeyError
        >>> MyEnum.loads('C')  # KeyError

        - Preprocessors

        >>> from enum import IntEnum, unique
        >>>
        >>> @int_enum_loads(name_preprocess=str.upper, value_preprocess=abs)
        >>> @unique
        >>> class MyEnum(IntEnum):
        >>>     A = 1
        >>>     B = 2
        >>>
        >>> MyEnum.loads(1)    # MyEnum.A
        >>> MyEnum.loads('A')  # MyEnum.A
        >>> MyEnum.loads(2)    # MyEnum.B
        >>> MyEnum.loads('B')  # MyEnum.B
        >>> MyEnum.loads(-1)   # MyEnum.A
        >>> MyEnum.loads('a')  # MyEnum.A
        >>> MyEnum.loads('C')  # KeyError

        - External processor

        >>> from enum import IntEnum, unique
        >>>
        >>> @int_enum_loads(external_process=lambda data: None)
        >>> @unique
        >>> class MyEnum(IntEnum):
        >>>     A = 1
        >>>     B = 2
        >>>
        >>> MyEnum.loads(1)    # MyEnum.A
        >>> MyEnum.loads('A')  # MyEnum.A
        >>> MyEnum.loads(2)    # MyEnum.B
        >>> MyEnum.loads('B')  # MyEnum.B
        >>> MyEnum.loads(-1)   # None
        >>> MyEnum.loads('a')  # None
        >>> MyEnum.loads('C')  # None
    """
    value_preprocess = value_preprocess or _default_value_preprocess
    name_preprocess = name_preprocess or _default_name_preprocess

    def _decorator(enum_class: Type[_EnumType]):
        if not issubclass(enum_class, IntEnum):
            raise TypeError('Int enum expected but {type} found.'.format(type=repr(enum_class.__name__)))
        enum_class = unique(enum_class)

        @lru_cache()
        def _dict_item():
            return {key: value for key, value in enum_class.__members__.items()}

        @lru_cache()
        def _int_value_to_item():
            return {value.value: value for _, value in _dict_item().items()}

        @lru_cache()
        def _str_name_to_item():
            return {name_preprocess(key): value for key, value in _dict_item().items()}

        def _load_func(data) -> Optional[enum_class]:
            if isinstance(data, enum_class):
                return data
            elif enable_int and isinstance(data, int):
                return _int_value_to_item()[value_preprocess(data)]
            elif enable_str and isinstance(data, str):
                return _str_name_to_item()[name_preprocess(data)]
            else:
                return (external_process or _get_default_external_preprocess(enum_class))(data)

        def loads(cls, data) -> Optional[enum_class]:
            """
            Overview:
                Load enum data from raw data.

            Arguments:
                - data (:obj:`Any`): Data which going to be parsed.

            Returns:
                - enum_data (:obj:): Parsed enum data
            """
            return _load_func(data)

        enum_class.loads = classmethod(loads)
        return enum_class

    return _decorator



from .clazz import init_magic, class_wraps, common_bases, common_direct_base
from .enum import int_enum_loads
from .final import FinalMeta
from .func import args_iter, dynamic_call
from .singleton import SingletonMeta, ValueBasedSingletonMeta, SingletonMark
from .tree import build_tree



from queue import Queue

from treelib import Tree as LibTree

_ROOT_ID = '_root'
_NODE_ID_TEMP = '_node_{id}'


def build_tree(root, represent=None, iterate=None, recurse=None) -> LibTree:
    """
    Overview:
        Build a treelib object by an object.

    Arguments:
        - root (:obj:`Any`): Root object.
        - represent (:obj:`Optional[Callable]`): Represent function, default is primitive `repr`.
        - iterate (:obj:`Optional[Callable]`): Iterate function, default is `lambda x: x.items()`.
        - recurse (:obj:`Optional[Callable]`): Recurse check function, default is `lambda x: hasattr(x, 'items')`.

    Returns:
        - tree (:obj:`treelib.Tree`): Built tree.

    Example:
         >>> t = build_tree(
         >>>     {'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}, 'z': [1, 2], 'v': {'1': '2'}},
         >>>     represent=lambda x: '<node>' if isinstance(x, dict) else repr(x),
         >>>     recurse=lambda x: isinstance(x, dict),
         >>> )
         >>> print(t)

         The output should be

         >>> <node>
         >>> ├── 'a' --> 1
         >>> ├── 'b' --> 2
         >>> ├── 'v' --> <node>
         >>> │   └── '1' --> '2'
         >>> ├── 'x' --> <node>
         >>> │   ├── 'c' --> 3
         >>> │   └── 'd' --> 4
         >>> └── 'z' --> [1, 2]
    """
    represent = represent or repr
    iterate = iterate or (lambda x: x.items())
    recurse = recurse or (lambda x: hasattr(x, 'items'))

    _tree = LibTree()
    _tree.create_node(represent(root), _ROOT_ID)
    _index, _queue = 0, Queue()
    _queue.put((_ROOT_ID, root))

    while not _queue.empty():
        _parent_id, _parent_tree = _queue.get()

        for key, value in iterate(_parent_tree):
            _index += 1
            _current_id = _NODE_ID_TEMP.format(id=_index)
            _tree.create_node(
                "{key} --> {value}".format(key=repr(key), value=represent(value)),
                _current_id,
                _parent_id
            )
            if recurse(value):
                _queue.put((_current_id, value))

    return _tree



from functools import partial, wraps, reduce
from operator import __and__
from queue import Queue
from typing import TypeVar, Type, Union, Collection, Set

_ClassType = TypeVar('_ClassType')


def class_wraps(original_class: Type[_ClassType]):
    """
    Overview:
        Wrap class like functools.wraps, can be used in class decorators.

    Arguments:
        - original_class (:obj:`Type[_ClassType]`): Original class for wrapping.

    Example:
        >>> def cls_dec(clazz):
        >>>     @class_wraps(clazz)
        >>>     class _NewClazz(clazz):
        >>>         pass
        >>>
        >>>     return _NewClazz
    """

    # noinspection PyTypeChecker
    def _decorator(clazz: Type[_ClassType]) -> Type[_ClassType]:
        new_clazz = type(original_class.__name__, (clazz,), {
            '__doc__': original_class.__doc__,
            '__module__': original_class.__module__,
        })
        return new_clazz

    return _decorator


def init_magic(init_decorator):
    """
    Overview:
        Magic for initialization function of a class.

    Arguments:
        - init_decorator (:obj:`Callable`): Initialization function decorator.

    Example:
        >>> from functools import wraps
        >>>
        >>> def _init_dec(func):
        >>>     @wraps(func)
        >>>     def _new_func(value):
        >>>         func(value + 1)
        >>>
        >>>     return _new_func
        >>>
        >>> @init_magic(_init_dec)
        >>> class Container:
        >>>     def __init__(self, value):
        >>>         self.__value = value
        >>>
        >>>     @property
        >>>     def value(self):
        >>>         return self.__value
        >>>
        >>> c = Container(1)
        >>> c.value   # 2
        >>> c2 = Container(33)
        >>> c2.value  # 34
    """

    def _decorator(clazz: Type[_ClassType]) -> Type[_ClassType]:
        @class_wraps(clazz)
        class _NewClass(clazz):
            @wraps(clazz.__init__)
            def __init__(self, *args, **kwargs):
                init_decorator(partial(clazz.__init__, self))(*args, **kwargs)

        return _NewClass

    return _decorator


def _get_all_bases(clazz: Type):
    queue = Queue()
    result = {clazz}
    queue.put(clazz)
    while not queue.empty():
        current = queue.get()
        for _type in current.__bases__:
            if _type not in result:
                result.add(_type)
                queue.put(_type)

    return result


def _get_all_direct_bases(clazz: Type):
    result = set()
    while True:
        result.add(clazz)
        clazz = clazz.__base__
        if clazz is None:
            break

    return result


def _base_process(base: Union[Collection[Type], Type, None]):
    if isinstance(base, type):
        base = (base,)
    elif base is not None:
        base = tuple(list(base))

    return base


def _all_commons(*classes: Type, func, base: Union[Collection[Type], Type, None] = None) -> Set[Type]:
    if not classes:
        return set()

    base = _base_process(base)
    _intersection = reduce(__and__, [func(item) for item in classes])
    _result = set()

    for _base_class in _intersection:
        acceptable = True
        for _another_base_class in _intersection:
            if _base_class == _another_base_class:
                continue

            if issubclass(_another_base_class, _base_class):
                acceptable = False
                break

        if acceptable:
            _result.add(_base_class)

    if base:
        _result = {item for item in _result if issubclass(item, base)}

    return _result


def common_bases(*classes: Type, base: Union[Collection[Type], Type, None] = None) -> Set[Type]:
    """
    Overview:
        Get all the common bases of the classes.

    Arguments:
        - classes (:obj:`Type`): Target classes.
        - base (:obj:`Union[Collection[Type], Type, None]`): Limit of the base class, \
            default is `None` which means no limit.

    Returns:
        - bases (:obj:`Set[Type]`): A set of classes which is the common bases of the classes, \
            only high leveled classes will be kept.
    """
    return _all_commons(*classes, func=_get_all_bases, base=base)


def common_direct_base(*classes: Type, base: Union[Collection[Type], Type, None] = None):
    """
    Overview:
        Get the common direct base of the classes.

    Arguments:
        - classes (:obj:`Type`): Target classes.
        - base (:obj:`Union[Collection[Type], Type, None]`): Limit of the base class, \
            default is `None` which means no limit.

    Returns:
        - base (:obj:`Type`): A class which is all the bases.
    """
    base = _base_process(base)
    _bases = _all_commons(*classes, func=_get_all_direct_bases, base=base)
    if _bases:
        return list(_bases)[0]
    else:
        if base:
            template = "No common base found with {classes} which is based on {bases}."
        else:
            template = "No common base found with {classes}."

        raise TypeError(template.format(classes=repr(classes), bases=repr(base)))



"""
Overview:
    Meta information for treevalue package.

.. data:: __TITLE__

    Title of this project (should be `treevalue`).

.. data:: __VERSION__

    Version of this project.

.. data:: __AUTHOR__

    Author of this project.

.. data:: __AUTHOR_EMAIL__

    Email of the author.
"""

__TITLE__ = "treevalue"
__VERSION__ = "0.0.1"
__AUTHOR__ = "HansBug, DI-engine's Contributors"
__AUTHOR_EMAIL__ = "hansbug@buaa.edu.cn"



import warnings
from functools import reduce

from .base import _BaseProcessor


class _OuterProcessor(_BaseProcessor):
    def get_key_set(self, *args, **kwargs):
        return reduce(lambda x, y: x | y, [set(keyset) for index, keyset in self._get_key_entries(*args, **kwargs)])

    def check_arguments(self, mode, return_type, inherit,
                        allow_missing, missing_func, subside, rise):
        _BaseProcessor.check_arguments(self, mode, return_type, inherit,
                                       allow_missing, missing_func, subside, rise)
        if not allow_missing:
            warnings.warn(RuntimeWarning("Missing is still not allowed, but this may cause KeyError in outer mode."))



from typing import List, Union, Callable, Any

from .base import BaseTree
from .tree import Tree


class TreeView(BaseTree):
    """
    Overview:
        Tree view data model, based on `BaseTree`.
    """

    def __init__(self, tree: Tree, path: List[str]):
        """
        Overview:
            Constructor of `TreeValue`.

        Arguments:
            - tree (:obj:`Tree`): Based tree.
            - path (:obj:`List[str]`): Viewing path.

        Example:
            >>> t = Tree({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
            >>> tv = t.view(["x"])                # tv should be equal to Tree({'c': 3, 'd': 4})
            >>> t['x'] = Tree({'c': 5, 'd': 6})
            >>> tv                                # tv should be equal to Tree({'c': 5, 'd': 6})
        """
        self.__tree = tree
        self.__path = [str(segment) for segment in path]

    def __get_actual_tree(self) -> BaseTree:
        _tree = self.__tree
        for _segment in self.__path:
            _tree = _tree[_segment]

        if not isinstance(_tree, BaseTree):
            raise TypeError(
                'Viewed target is not a tree but {target} found.'.format(target=repr(type(_tree.__class__))))
        else:
            return _tree

    def __getitem__(self, key):
        return self.__get_actual_tree().__getitem__(key)

    def __setitem__(self, key, value):
        self.__get_actual_tree().__setitem__(key, value)

    def __delitem__(self, key):
        self.__get_actual_tree().__delitem__(key)

    def view(self, path: List[str]):
        return TreeView(self.__tree, self.__path + [str(segment) for segment in path])

    def clone(self, copy_value: Union[None, bool, Callable, Any] = None):
        return self.__get_actual_tree().clone(copy_value)

    def items(self):
        return self.__get_actual_tree().items()

    def keys(self):
        return self.__get_actual_tree().keys()

    def values(self):
        return self.__get_actual_tree().values()

    def actual(self):
        return self.__get_actual_tree().actual()



from functools import lru_cache
from typing import List, Mapping, Optional, Any, Type, TypeVar, Union, Callable

from ..func import method_treelize
from ..tree import TreeValue, jsonify, view, clone, typetrans, mapping, mask, filter_, reduce_, union, subside, rise, \
    NO_RISE_TEMPLATE

_BASE_GENERATION_CONFIG = {}


def general_tree_value(base: Optional[Mapping[str, Any]] = None,
                       methods: Optional[Mapping[str, Optional[Mapping[str, Any]]]] = None):
    """
    Overview:
        Get general tree value class.

    Arguments:
        - base (:obj:`Optional[Mapping[str, Any]]`): Base configuration of `func_treelize`.
        - methods (:obj:`Optional[Mapping[str, Optional[Mapping[str, Any]]]]`): Method configurations of `func_treelize`.

    Returns:
        - clazz (:obj:`Type[TreeValue]`): General tree value class.
    """
    base = base or {}
    methods = methods or {}

    @lru_cache()
    def _decorator_config(name):
        _config = _BASE_GENERATION_CONFIG.copy()
        _config.update(base)
        _config.update(methods.get(name, None) or {})
        return _config

    def _decorate(func):
        return method_treelize(**_decorator_config(func.__name__))(func)

    _TreeValue = TypeVar("_TreeValue", bound=TreeValue)

    # noinspection PyUnresolvedReferences
    class _GeneralTreeValue(TreeValue):
        @method_treelize()
        def _attr_extern(self, key):
            """
            Overview:
                External protected function for support the unfounded attributes. \
                In `FastTreeValue` it is extended to support the access to values' attribute.

            Arguments:
                - key (:obj:`str`): Attribute name.

            Returns:
                - return (:obj:): TreeValue of the values' attribute.

            Example:
                >>> class Container:
                >>>     def __init__(self, value):
                >>>         self.__value = value
                >>>
                >>>     @property
                >>>     def value(self):
                >>>         return self.__value
                >>>
                >>>     def append(self, v):
                >>>         return self.__value + v
                >>>
                >>> t = FastTreeValue({'a': Container(1), 'b': Container(2)})
                >>> t.value   # FastTreeValue({'a': 1, 'b': 2})
                >>> t.append  # FastTreeValue({'a': <method 'append' of Container(1)>, 'b': <method 'append' of Container(2)>})
            """
            return getattr(self, key)

        def json(self):
            """
            Overview:
                Dump current `TreeValue` object to json data.

            Returns:
                - json (:obj:`dict`): Dumped json data.

            Example:
                >>> t = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> t.json()  # {'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}}
            """
            return jsonify(self)

        def view(self, path: List[str]):
            """
            Overview:
                Create a `TreeValue` object which is a view of the current tree.

            Arguments:
                - path (:obj:`List[str]`): Path of the view.

            Returns:
                - tree (:obj:`_TreeValue`): Viewed tree value object.

            Example:
                >>> t = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> t.view(['x'])  # FastTreeValue({'c': 3, 'd': 4}), it is a view not the real node
            """
            return view(self, path)

        def clone(self, copy_value: Union[None, bool, Callable, Any] = None):
            """
            Overview:
                Create a fully clone of the current tree.

            Returns:
                - tree (:obj:`_TreeValue`): Cloned tree value object.
                - copy_value (:obj:`Union[None, bool, Callable, Any]`): Deep copy value or not, \
                    default is `None` which means do not deep copy the values. \
                    If deep copy is required, just set it to `True`.

            Example:
                >>> t = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> t.x.clone()  # FastTreeValue({'c': 3, 'd': 4})
            """
            return clone(self, copy_value)

        def type(self, clazz: Type[_TreeValue]) -> _TreeValue:
            """
            Overview:
                Transform tree value object to another tree value type. \
                Attention that in this function, no copy will be made, \
                the original tree value and the transformed tree value are using the same space area.

            Arguments:
                - return_type (:obj:`Type[_TreeValue]`): Target tree value type

            Returns:
                - tree (:obj:`_TreeValue`): Transformed tree value object.

            Example:
                >>> t = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> t.type(TreeValue)  # TreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
            """
            return typetrans(self, clazz)

        def map(self, mapper):
            """
            Overview:
                Do mapping on every value in this tree.

            Arguments:
                - func (:obj:): Function for mapping

            Returns:
                - tree (:obj:`_TreeValue`): Mapped tree value object.

            Example:
                >>> t = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> t.map(lambda x: x + 2)  # FastTreeValue({'a': 3, 'b': 4, 'x': {'c': 5, 'd': 6}})
                >>> t.map(lambda: 1)        # FastTreeValue({'a': 1, 'b': 1, 'x': {'c': 1, 'd': 1}})
                >>> t.map(lambda x, p: p)   # FastTreeValue({'a': ('a',), 'b': ('b',), 'x': {'c': ('x', 'c'), 'd': ('x', 'd')}})
            """
            return mapping(self, mapper)

        def mask(self, mask_: TreeValue, remove_empty: bool = True):
            """
            Overview:
                Filter the element in the tree with a mask

            Arguments:
                - `mask_` (:obj:`TreeValue`): Tree value mask object
                - `remove_empty` (:obj:`bool`): Remove empty tree node automatically, default is `True`.

            Returns:
                - tree (:obj:`_TreeValue`): Filtered tree value object.

            Example:
                >>> t = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> t.mask(TreeValue({'a': True, 'b': False, 'x': False}))                    # FastTreeValue({'a': 1})
                >>> t.mask(TreeValue({'a': True, 'b': False, 'x': {'c': True, 'd': False}}))  # FastTreeValue({'a': 1, 'x': {'c': 3}})
            """
            return mask(self, mask_, remove_empty)

        def filter(self, func, remove_empty: bool = True):
            """
            Overview:
                Filter the element in the tree with a predict function.

            Arguments:
                - func (:obj:): Function for filtering
                - remove_empty (:obj:`bool`): Remove empty tree node automatically, default is `True`.

            Returns:
                - tree (:obj:`_TreeValue`): Filtered tree value object.

            Example:
                >>> t = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> t.filter(lambda x: x < 3)                  # FastTreeValue({'a': 1, 'b': 2})
                >>> t.filter(lambda x: x < 3, False)           # FastTreeValue({'a': 1, 'b': 2, 'x': {}})
                >>> t.filter(lambda x: x % 2 == 1)             # FastTreeValue({'a': 1, 'x': {'c': 3}})
                >>> t.filter(lambda x, p: p[0] in {'b', 'x'})  # FastTreeValue({'b': 2, 'x': {'c': 3, 'd': 4}})
            """
            return filter_(self, func, remove_empty)

        def reduce(self, func):
            """
            Overview
                Reduce the tree to value.

            Arguments:
                - func (:obj:): Function for reducing

            Returns:
                - result (:obj:): Reduce result

            Examples:
                >>> from functools import reduce
                >>>
                >>> t = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> t.reduce(lambda **kwargs: sum(kwargs.values()))  # 10, 1 + 2 + (3 + 4)
                >>> t.reduce(lambda **kwargs: reduce(lambda x, y: x * y, list(kwargs.values())))  # 24, 1 * 2 * (3 * 4)
            """
            return reduce_(self, func)

        def rise(self, dict_: bool = True, list_: bool = True, tuple_: bool = True,
                 template=NO_RISE_TEMPLATE):
            """
            Overview:
                Make the structure (dict, list, tuple) in value rise up to the top, above the tree value.

            Arguments:
                - `dict_` (:obj:`bool`): Enable dict rise, default is `True`.
                - `list_` (:obj:`bool`): Enable list rise, default is `True`.
                - `tuple_` (:obj:`bool`): Enable list rise, default is `True`.
                - template (:obj:): Rising template, default is `NO_RISE_TEMPLATE`, which means auto detect.

            Returns:
                - risen (:obj:): Risen value.

            Example:
                >>> t = FastTreeValue({'x': raw({'a': [1, 2], 'b': [2, 3]}), 'y': raw({'a': [5, 6, 7], 'b': [7, 8]})})
                >>> dt = t.rise()
                >>> # dt will be {'a': <FastTreeValue 1>, 'b': [<FastTreeValue 2>, <FastTreeValue 3>]}
                >>> # FastTreeValue 1 will be FastTreeValue({'x': [1, 2], 'y': [5, 6, 7]})
                >>> # FastTreeValue 2 will be FastTreeValue({'x': 2, 'y': 7})
                >>> # FastTreeValue 3 will be FastTreeValue({'x': 3, 'y': 8})
                >>>
                >>> t2 = FastTreeValue({'x': raw({'a': [1, 2], 'b': [2, 3]}), 'y': raw({'a': [5, 6], 'b': [7, 8]})})
                >>> dt2 = t2.rise()
                >>> # dt2 will be {'a': [<FastTreeValue 1>, <FastTreeValue 2>], 'b': [<FastTreeValue 3>, <FastTreeValue 4>]}
                >>> # FastTreeValue 1 will be FastTreeValue({'x': 1, 'y': 5})
                >>> # FastTreeValue 2 will be FastTreeValue({'x': 2, 'y': 6})
                >>> # FastTreeValue 3 will be FastTreeValue({'x': 2, 'y': 7})
                >>> # FastTreeValue 4 will be FastTreeValue({'x': 3, 'y': 8})
                >>>
                >>> dt3 = t2.rise(template={'a': None, 'b': None})
                >>> # dt3 will be {'a': <FastTreeValue 1>, 'b': <FastTreeValue 2>}
                >>> # FastTreeValue 1 will be FastTreeValue({'x': [1, 2], 'y': [5, 6]})
                >>> # FastTreeValue 2 will be FastTreeValue({'x': [2, 3], 'y': [7, 8]})
            """
            return rise(self, dict_, list_, tuple_)

        @classmethod
        def union(cls, *trees, return_type=None, **kwargs):
            """
            Overview:
                Union tree values together.

            Arguments:
                - trees (:obj:`_TreeValue`): Tree value objects.
                - mode (:obj:): Mode of the wrapping (string or TreeMode both okay), default is `strict`.
                - return_type (:obj:`Optional[Type[_ClassType]]`): Return type of the wrapped function, default is `TreeValue`.
                - inherit (:obj:`bool`): Allow inherit in wrapped function, default is `True`.
                - missing (:obj:): Missing value or lambda generator of when missing, default is `MISSING_NOT_ALLOW`, which \
                    means raise `KeyError` when missing detected.

            Returns:
                - result (:obj:`TreeValue`): Unionised tree value.

            Example:
                >>> t = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> tx = t.map(lambda v: v % 2 == 1)
                >>> FastTreeValue.union(t, tx)  # FastTreeValue({'a': (1, True), 'b': (2, False), 'x': {'c': (3, True), 'd': (4, False)}})
            """
            return union(*trees, return_type=return_type or cls, **kwargs)

        @classmethod
        def subside(cls, value, dict_: bool = True, list_: bool = True, tuple_: bool = True,
                    return_type: Optional[Type[_TreeValue]] = None, **kwargs):
            """
            Overview:
                Drift down the structures (list, tuple, dict) down to the tree's value.

            Arguments:
                - value (:obj:): Original value object, may be nested dict, list or tuple.
                - `dict_` (:obj:`bool`): Enable dict subside, default is `True`.
                - `list_` (:obj:`bool`): Enable list subside, default is `True`.
                - `tuple_` (:obj:`bool`): Enable list subside, default is `True`.
                - mode (:obj:): Mode of the wrapping (string or TreeMode both okay), default is `strict`.
                - return_type (:obj:`Optional[Type[_ClassType]]`): Return type of the wrapped function, \
                    will be auto detected when there is exactly one tree value type in this original value, \
                    otherwise the default will be `TreeValue`.
                - inherit (:obj:`bool`): Allow inherit in wrapped function, default is `True`.
                - missing (:obj:): Missing value or lambda generator of when missing, default is `MISSING_NOT_ALLOW`, which \
                    means raise `KeyError` when missing detected.

            Returns:
                - return (:obj:`_TreeValue`): Subsided tree value.

            Example:
                >>> data = {
                >>>     'a': TreeValue({'a': 1, 'b': 2}),
                >>>     'x': {
                >>>         'c': TreeValue({'a': 3, 'b': 4}),
                >>>         'd': [
                >>>             TreeValue({'a': 5, 'b': 6}),
                >>>             TreeValue({'a': 7, 'b': 8}),
                >>>         ]
                >>>     },
                >>>     'k': '233'
                >>> }
                >>>
                >>> tree = FastTreeValue.subside(data)
                >>> # tree should be --> FastTreeValue({
                >>> #    'a': raw({'a': 1, 'k': '233', 'x': {'c': 3, 'd': [5, 7]}}),
                >>> #    'b': raw({'a': 2, 'k': '233', 'x': {'c': 4, 'd': [6, 8]}}),
                >>> #}), all structures above the tree values are subsided to the bottom of the tree.
            """
            return subside(value, dict_, list_, tuple_,
                           return_type=return_type or cls, **kwargs)

        @_decorate
        def __add__(self, other):
            """
            Overview:
                Add tree values together.

            Example:
                >>> t1 = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> t2 = FastTreeValue({'a': 11, 'b': 22, 'x': {'c': 30, 'd': 40}})
                >>> t1 + t2  # FastTreeValue({'a': 12, 'b': 24, 'x': {'c': 33, 'd': 44}})
            """
            return self + other

        @_decorate
        def __radd__(self, other):
            """
            Overview:
                Right version of `__add__`.

            Example:
                >>> t1 = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> 1 + t1  # FastTreeValue({'a': 2, 'b': 3, 'x': {'c': 4, 'd': 5}})
            """
            return other + self

        @_decorate
        def __sub__(self, other):
            """
            Overview:
                Substract tree values.

            Example:
                >>> t1 = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> t2 = FastTreeValue({'a': 11, 'b': 22, 'x': {'c': 30, 'd': 40}})
                >>> t1 - t2  # FastTreeValue({'a': -10, 'b': -20, 'x': {'c': -27, 'd': -36}})
            """
            return self - other

        @_decorate
        def __rsub__(self, other):
            """
            Overview:
                Right version of `__sub__`.

            Example:
                >>> t1 = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> 1 - t1  # FastTreeValue({'a': 0, 'b': -1, 'x': {'c': -2, 'd': -3}})
            """
            return other - self

        @_decorate
        def __mul__(self, other):
            """
            Overview:
                Multiply tree values together.

            Example:
                >>> t1 = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> t2 = FastTreeValue({'a': 11, 'b': 22, 'x': {'c': 30, 'd': 40}})
                >>> t1 * t2  # FastTreeValue({'a': 11, 'b': 44, 'x': {'c': 90, 'd': 160}})
            """
            return self * other

        @_decorate
        def __rmul__(self, other):
            """
            Overview:
                Right version of `__mul__`.

            Example:
                >>> t1 = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> 2 * t1  # FastTreeValue({'a': 2, 'b': 4, 'x': {'c': 6, 'd': 8}})
            """
            return other * self

        @_decorate
        def __matmul__(self, other):
            """
            Overview:
                Matrix tree values together, can be used in numpy or torch.
            """
            return self @ other

        @_decorate
        def __rmatmul__(self, other):
            """
            Overview:
                Right version of `__matmul__`.
            """
            return other @ self

        @_decorate
        def __truediv__(self, other):
            """
            Overview:
                True divide tree values.

            Example:
                >>> t1 = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> t2 = FastTreeValue({'a': 10, 'b': 25, 'x': {'c': 30, 'd': 40}})
                >>> t1 / t2  # FastTreeValue({'a': 0.1, 'b': 0.08, 'x': {'c': 0.1, 'd': 0.1}})
            """
            return self / other

        @_decorate
        def __rtruediv__(self, other):
            """
            Overview:
                Right version of `__truediv__`.

            Example:
                >>> t1 = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> 6 / t1  # FastTreeValue({'a': 6, 'b': 3, 'x': {'c': 2, 'd': 1.5}})
            """
            return other / self

        @_decorate
        def __floordiv__(self, other):
            """
            Overview:
                Floor divide tree values.

            Example:
                >>> t1 = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> t2 = FastTreeValue({'a': 10, 'b': 25, 'x': {'c': 30, 'd': 40}})
                >>> t2 // t1  # FastTreeValue({'a': 10, 'b': 12, 'x': {'c': 10, 'd': 10}})
            """
            return self // other

        @_decorate
        def __rfloordiv__(self, other):
            """
            Overview:
                Right version of `__floordiv__`.

            Example:
                >>> t1 = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> 6 // t1  # FastTreeValue({'a': 6, 'b': 3, 'x': {'c': 2, 'd': 1}})
            """
            return other // self

        @_decorate
        def __mod__(self, other):
            """
            Overview:
                Mod tree values.

            Example:
                >>> t1 = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> t2 = FastTreeValue({'a': 10, 'b': 25, 'x': {'c': 30, 'd': 40}})
                >>> t2 % t1  # FastTreeValue({'a': 0, 'b': 1, 'x': {'c': 0, 'd': 0}})
            """
            return self % other

        @_decorate
        def __rmod__(self, other):
            """
            Overview:
                Right version of `__mod__`.

            Example:
                >>> t1 = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> 6 % t1  # FastTreeValue({'a': 0, 'b': 0, 'x': {'c': 0, 'd': 2}})
            """
            return other % self

        @_decorate
        def __pow__(self, power):
            """
            Overview:
                Mod tree values.

            Example:
                >>> t1 = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> t2 = FastTreeValue({'a': 2, 'b': 3, 'x': {'c': 4, 'd': 5}})
                >>> t1 ** t2  # FastTreeValue({'a': 1, 'b': 8, 'x': {'c': 81, 'd': 1024}})
            """
            return self ** power

        @_decorate
        def __rpow__(self, other):
            """
            Overview:
                Right version of `__pow__`.

            Example:
                >>> t1 = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> 2 ** t1  # FastTreeValue({'a': 2, 'b': 4, 'x': {'c': 8, 'd': 16}})
            """
            return other ** self

        @_decorate
        def __and__(self, other):
            """
            Overview:
                Bitwise and tree values.

            Example:
                >>> t1 = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> t2 = FastTreeValue({'a': 2, 'b': 3, 'x': {'c': 4, 'd': 5}})
                >>> t1 & t2  # FastTreeValue({'a': 0, 'b': 2, 'x': {'c': 0, 'd': 4}})
            """
            return self & other

        @_decorate
        def __rand__(self, other):
            """
            Overview:
                Right version of `__and__`.

            Example:
                >>> t1 = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> 5 & t1  # FastTreeValue({'a': 1, 'b': 0, 'x': {'c': 1, 'd': 4}})
            """
            return other & self

        @_decorate
        def __or__(self, other):
            """
            Overview:
                Bitwise or tree values.

            Example:
                >>> t1 = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> t2 = FastTreeValue({'a': 2, 'b': 3, 'x': {'c': 4, 'd': 5}})
                >>> t1 | t2  # FastTreeValue({'a': 3, 'b': 3, 'x': {'c': 7, 'd': 5}})
            """
            return self | other

        @_decorate
        def __ror__(self, other):
            """
            Overview:
                Right version of `__or__`.

            Example:
                >>> t1 = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> 5 | t1  # FastTreeValue({'a': 5, 'b': 7, 'x': {'c': 7, 'd': 5}})
            """
            return other | self

        @_decorate
        def __xor__(self, other):
            """
            Overview:
                Bitwise or tree values.

            Example:
                >>> t1 = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> t2 = FastTreeValue({'a': 2, 'b': 3, 'x': {'c': 4, 'd': 5}})
                >>> t1 ^ t2  # FastTreeValue({'a': 3, 'b': 1, 'x': {'c': 7, 'd': 1}})
            """
            return self ^ other

        @_decorate
        def __rxor__(self, other):
            """
            Overview:
                Right version of `__xor__`.

            Example:
                >>> t1 = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> 5 ^ t1  # FastTreeValue({'a': 4, 'b': 7, 'x': {'c': 6, 'd': 1}})
            """
            return other ^ self

        @_decorate
        def __lshift__(self, other):
            """
            Overview:
                Left shift tree values.

            Example:
                >>> t1 = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> t2 = FastTreeValue({'a': 2, 'b': 3, 'x': {'c': 4, 'd': 5}})
                >>> t1 << t2  # FastTreeValue({'a': 4, 'b': 16, 'x': {'c': 48, 'd': 128}})
            """
            return self << other

        @_decorate
        def __rlshift__(self, other):
            """
            Overview:
                Right version of `__lshift__`.

            Example:
                >>> t1 = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> 3 << t1  # FastTreeValue({'a': 6, 'b': 12, 'x': {'c': 24, 'd': 48}})
            """
            return other << self

        @_decorate
        def __rshift__(self, other):
            """
            Overview:
                Left shift tree values.

            Example:
                >>> t1 = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> t2 = FastTreeValue({'a': 20, 'b': 30, 'x': {'c': 40, 'd': 50}})
                >>> t2 >> t1  # FastTreeValue({'a': 10, 'b': 7, 'x': {'c': 5, 'd': 3}})
            """
            return self >> other

        @_decorate
        def __rrshift__(self, other):
            """
            Overview:
                Right version of `__rshift__`.

            Example:
                >>> t1 = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> 64 >> t1  # FastTreeValue({'a': 32, 'b': 16, 'x': {'c': 8, 'd': 4}})
            """
            return other >> self

        @_decorate
        def __pos__(self):
            """
            Overview:
                Positive tree values.

            Examples:
                >>> t1 = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> +t1  # FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
            """
            return +self

        @_decorate
        def __neg__(self):
            """
            Overview:
                Negative tree values.

            Examples:
                >>> t1 = FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> -t1  # FastTreeValue({'a': -1, 'b': -2, 'x': {'c': -3, 'd': -4}})
            """
            return -self

        @_decorate
        def __invert__(self):
            """
            Overview:
                Bitwise invert tree values.

            Examples:
                >>> t1 = FastTreeValue({'a': 1, 'b': -2, 'x': {'c': 3, 'd': -4}})
                >>> ~t1  # FastTreeValue({'a': -2, 'b': 1, 'x': {'c': -4, 'd': 3}})
            """
            return ~self

        @_decorate
        def __getitem__(self, item):
            """
            Overview:
                Get item of tree values.

            Examples:
                >>> t1 = FastTreeValue({'a': [1, 2], 'b': [2, 3], 'x': {'c': [3, 4], 'd': [4, 5]}})
                >>> t1[0]     # FastTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
                >>> t1[-1]    # FastTreeValue({'a': 2, 'b': 3, 'x': {'c': 4, 'd': 5}})
                >>> t1[::-1]  # FastTreeValue({'a': [2, 1], 'b': [3, 2], 'x': {'c': [4, 3], 'd': [5, 4]}})
            """
            return self[item]

        @_decorate
        def __setitem__(self, key, value):
            """
            Overview:
                Set item of tree values.

            Examples:
                >>> t1 = FastTreeValue({'a': [1, 2], 'b': [2, 3], 'x': {'c': [3, 4], 'd': [4, 5]}})
                >>> t1[0] = -2  # FastTreeValue({'a': [-2, 2], 'b': [-2, 3], 'x': {'c': [-2, 4], 'd': [-2, 5]}})
                >>> t1[0] = FastTreeValue({'a': 2, 'b': 3, 'x': {'c': 4, 'd': 5}})
                >>> # FastTreeValue({'a': [2, 2], 'b': [3, 3], 'x': {'c': [4, 4], 'd': [5, 5]}})
            """
            self[key] = value

        @_decorate
        def __delitem__(self, key):
            """
            Overview:
                Delete item of tree values.

            Examples:
                >>> t1 = FastTreeValue({'a': [1, 2], 'b': [2, 3], 'x': {'c': [3, 4], 'd': [4, 5]}})
                >>> del t1[0]  # FastTreeValue({'a': [2], 'b': [3], 'x': {'c': [4], 'd': [5]}})
            """
            del self[key]

        @_decorate
        def __call__(self, *args, **kwargs):
            """
            Overview:
                Call of tree values.

            Example:
                >>> class Container:
                >>>     def __init__(self, value):
                >>>         self.__value = value
                >>>
                >>>     def append(self, v):
                >>>         return self.__value + v
                >>>
                >>> t = FastTreeValue({'a': Container(1), 'b': Container(2)})
                >>> t.append(2)  # FastTreeValue({'a': 3, 'b': 4})
                >>> t.append(FastTreeValue({'a': 10, 'b': 20}))  # FastTreeValue({'a': 11, 'b': 22})
            """
            return self(*args, **kwargs)

    return _GeneralTreeValue



class SingletonMeta(type):
    """
    Overview:
        Meta class for singleton mode.

    Example:
        >>> class MyService(metaclass=SingletonMeta):
        >>>     def get_value(self):
        >>>         return 233
        >>>
        >>> s = MyService()
        >>> s.get_value()    # 233
        >>> s1 = MyService()
        >>> s1 is s          # True
    """
    _instances = {}

    def __call__(cls):
        if cls not in cls._instances:
            cls._instances[cls] = super(SingletonMeta, cls).__call__()
        return cls._instances[cls]


class ValueBasedSingletonMeta(type):
    """
    Overview:
        Meta class for value based singleton mode.

    Example:
        >>> class MyData(metaclass=ValueBasedSingletonMeta):
        >>>     def __init__(self, value):
        >>>         self.__value = value
        >>>
        >>>     @property
        >>>     def value(self):
        >>>         return self.__value
        >>>
        >>> d1 = MyData(1)
        >>> d1.value       # 1
        >>> d2 = MyData(1)
        >>> d3 = MyData(2)
        >>> d2 is d1       # True
        >>> d2 is d3       # False
    """
    _instances = {}

    def __call__(cls, value):
        key = (cls, value)
        if key not in cls._instances:
            cls._instances[key] = super(ValueBasedSingletonMeta, cls).__call__(value)
        return cls._instances[key]


class SingletonMark(metaclass=ValueBasedSingletonMeta):
    """
    Overview:
        Singleton mark for some situation.
        Can be used when some default value is needed, especially when `None` has meaning which is not default.

    Example:
        >>> NO_VALUE = SingletonMark("no_value")
        >>> NO_VALUE is SingletonMark("no_value")  # True
    """

    def __init__(self, mark):
        self.__mark = mark

    @property
    def mark(self):
        """
        Overview:
            Get mark string of this mark object.

        Returns:
            - mark (:obj:`str`): Mark string
        """
        return self.__mark

    def __hash__(self):
        return hash(self.__mark)

    def __eq__(self, other):
        if other is self:
            return True
        elif type(other) == type(self):
            return other.__mark == self.__mark
        else:
            return False

    def __repr__(self):
        return "<{cls} {mark}>".format(
            cls=self.__class__.__name__,
            mark=repr(self.__mark),
        )



from functools import partial
from itertools import chain
from typing import TypeVar, List, Type, Tuple, Union, Any, Optional, Callable

from .tree import TreeValue, get_data_property
from ..common import raw
from ...utils import dynamic_call, common_direct_base, SingletonMark

_TreeValue = TypeVar("_TreeValue", bound=TreeValue)


def jsonify(tree: _TreeValue):
    """
    Overview:
        Dump `TreeValue` object to json data.

    Arguments:
        - tree (:obj:`_TreeValue`): Tree value object.

    Returns:
        - json (:obj:`dict`): Dumped json data.

    Example:
        >>> jsonify(TreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}}))  # {'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}}
    """
    return get_data_property(tree).json()


def view(tree: _TreeValue, path: List[str]) -> _TreeValue:
    """
    Overview:
        Create a `TreeValue` object which is a view of given tree.

    Arguments:
        - tree (:obj:`_TreeValue`): Tree value object.
        - path (:obj:`List[str]`): Path of the view.

    Returns:
        - tree (:obj:`_TreeValue`): Viewed tree value object.

    Example:
        >>> view(TreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}}), ['x'])  # TreeValue({'c': 3, 'd': 4})
    """
    return tree.__class__(get_data_property(tree).view(path))


def clone(tree: _TreeValue, copy_value: Union[None, bool, Callable, Any] = None) -> _TreeValue:
    """
    Overview:
        Create a fully clone of the given tree.

    Arguments:
        - tree (:obj:`_TreeValue`): Tree value object
        - copy_value (:obj:`Union[None, bool, Callable, Any]`): Deep copy value or not, \
            default is `None` which means do not deep copy the values. \
            If deep copy is required, just set it to `True`.

    Returns:
        - tree (:obj:`_TreeValue`): Cloned tree value object.

    Example:
        >>> t = TreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
        >>> clone(t.x)  # TreeValue({'c': 3, 'd': 4})
    """
    return tree.__class__(get_data_property(tree).clone(copy_value))


def typetrans(tree: TreeValue, return_type: Type[_TreeValue]) -> _TreeValue:
    """
    Overview:
        Transform tree value object to another tree value type. \
        Attention that in this function, no copy will be made, \
        the original tree value and the transformed tree value are using the same space area.

    Arguments:
        - tree (:obj:`TreeValue`): Tree value object
        - return_type (:obj:`Type[_TreeValue]`): Target tree value type

    Returns:
        - tree (:obj:`_TreeValue`): Transformed tree value object.

    Example:
        >>> t = MyTreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
        >>> typetrans(t, TreeValue)  # TreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
    """
    if not issubclass(return_type, TreeValue):
        raise TypeError("Tree value should be subclass of TreeValue, but {type} found.".format(
            type=repr(return_type.__name__)
        ))

    return return_type(get_data_property(tree))


def _build_path_tree(tree: _TreeValue) -> _TreeValue:
    def _recursion(path, t: _TreeValue):
        if isinstance(t, tree.__class__):
            return tree.__class__({key: _recursion(path + [key], value) for key, value in t})
        else:
            return tuple(path)

    return _recursion([], tree)


def mapping(tree: _TreeValue, func: Callable) -> _TreeValue:
    """
    Overview:
        Do mapping on every value in this tree.

    Arguments:
        - tree (:obj:`_TreeValue`): Tree value object
        - func (:obj:`Callable`): Function for mapping

    Returns:
        - tree (:obj:`_TreeValue`): Mapped tree value object.

    Example:
        >>> t = TreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
        >>> mapping(t, lambda x: x + 2)  # TreeValue({'a': 3, 'b': 4, 'x': {'c': 5, 'd': 6}})
        >>> mapping(t, lambda: 1)        # TreeValue({'a': 1, 'b': 1, 'x': {'c': 1, 'd': 1}})
        >>> mapping(t, lambda x, p: p)   # TreeValue({'a': ('a',), 'b': ('b',), 'x': {'c': ('x', 'c'), 'd': ('x', 'd')}})
    """

    from ..func import func_treelize
    return func_treelize(return_type=tree.__class__)(lambda args: dynamic_call(func)(*args))(
        union(tree, _build_path_tree(tree)))


def _filter_by_masked_tree(masked_tree: _TreeValue, remove_empty: bool) -> _TreeValue:
    def _recursion(t: Union[_TreeValue, Any]) -> Tuple[bool, _TreeValue]:
        if isinstance(t, masked_tree.__class__):
            dict_result = {key: _recursion(value) for key, value in t}
            dict_result = {key: value for key, (flag, value) in dict_result.items() if flag}
            result = masked_tree.__class__(dict_result)

            return not not result if remove_empty else True, result
        else:
            return t

    _, _result = _recursion(masked_tree)
    return _result


def mask(tree: _TreeValue, mask_: Union[TreeValue, bool], remove_empty: bool = True) -> _TreeValue:
    """
    Overview:
        Filter the element in the tree with a mask

    Arguments:
        - `tree` (:obj:`_TreeValue`): Tree value object
        - `mask_` (:obj:`TreeValue`): Tree value mask object
        - `remove_empty` (:obj:`bool`): Remove empty tree node automatically, default is `True`.

    Returns:
        - tree (:obj:`_TreeValue`): Filtered tree value object.

    Example:
        >>> t = TreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
        >>> mask(t, TreeValue({'a': True, 'b': False, 'x': False}))                    # TreeValue({'a': 1})
        >>> mask(t, TreeValue({'a': True, 'b': False, 'x': {'c': True, 'd': False}}))  # TreeValue({'a': 1, 'x': {'c': 3}})
    """
    return _filter_by_masked_tree(union(mask_, tree, return_type=tree.__class__), remove_empty)


def filter_(tree: _TreeValue, func: Callable, remove_empty: bool = True) -> _TreeValue:
    """
    Overview:
        Filter the element in the tree with a predict function.

    Arguments:
        - tree (:obj:`_TreeValue`): Tree value object
        - func (:obj:`Callable`): Function for filtering
        - remove_empty (:obj:`bool`): Remove empty tree node automatically, default is `True`.

    Returns:
        - tree (:obj:`_TreeValue`): Filtered tree value object.

    Example:
        >>> t = TreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
        >>> filter_(t, lambda x: x < 3)                  # TreeValue({'a': 1, 'b': 2})
        >>> filter_(t, lambda x: x < 3, False)           # TreeValue({'a': 1, 'b': 2, 'x': {}})
        >>> filter_(t, lambda x: x % 2 == 1)             # TreeValue({'a': 1, 'x': {'c': 3}})
        >>> filter_(t, lambda x, p: p[0] in {'b', 'x'})  # TreeValue({'b': 2, 'x': {'c': 3, 'd': 4}})
    """
    return mask(tree, mapping(tree, func), remove_empty)


def union(*trees: TreeValue, return_type=None, **kwargs):
    """
    Overview:
        Union tree values together.

    Arguments:
        - trees (:obj:`_TreeValue`): Tree value objects.
        - mode (:obj:): Mode of the wrapping (string or TreeMode both okay), default is `strict`.
        - return_type (:obj:`Optional[Type[_ClassType]]`): Return type of the wrapped function, default is `TreeValue`.
        - inherit (:obj:`bool`): Allow inherit in wrapped function, default is `True`.
        - missing (:obj:): Missing value or lambda generator of when missing, default is `MISSING_NOT_ALLOW`, which \
            means raise `KeyError` when missing detected.

    Returns:
        - result (:obj:`TreeValue`): Unionised tree value.

    Example:
        >>> t = TreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
        >>> tx = mapping(t, lambda v: v % 2 == 1)
        >>> union(t, tx)  # TreeValue({'a': (1, True), 'b': (2, False), 'x': {'c': (3, True), 'd': (4, False)}})
    """
    if return_type is None:
        return_type = trees[0].__class__ if trees else TreeValue

    return subside(tuple(trees), return_type=return_type, **kwargs)


def subside(value, dict_: bool = True, list_: bool = True, tuple_: bool = True,
            return_type: Optional[Type[_TreeValue]] = None, **kwargs) -> _TreeValue:
    """
    Overview:
        Drift down the structures (list, tuple, dict) down to the tree's value.

    Arguments:
        - value (:obj:): Original value object, may be nested dict, list or tuple.
        - `dict_` (:obj:`bool`): Enable dict subside, default is `True`.
        - `list_` (:obj:`bool`): Enable list subside, default is `True`.
        - `tuple_` (:obj:`bool`): Enable list subside, default is `True`.
        - mode (:obj:): Mode of the wrapping (string or TreeMode both okay), default is `strict`.
        - return_type (:obj:`Optional[Type[_ClassType]]`): Return type of the wrapped function, \
            will be auto detected when there is exactly one tree value type in this original value, \
            otherwise the default will be `TreeValue`.
        - inherit (:obj:`bool`): Allow inherit in wrapped function, default is `True`.
        - missing (:obj:): Missing value or lambda generator of when missing, default is `MISSING_NOT_ALLOW`, which \
            means raise `KeyError` when missing detected.

    Returns:
        - return (:obj:`_TreeValue`): Subsided tree value.

    Example:
        >>> data = {
        >>>     'a': TreeValue({'a': 1, 'b': 2}),
        >>>     'x': {
        >>>         'c': TreeValue({'a': 3, 'b': 4}),
        >>>         'd': [
        >>>             TreeValue({'a': 5, 'b': 6}),
        >>>             TreeValue({'a': 7, 'b': 8}),
        >>>         ]
        >>>     },
        >>>     'k': '233'
        >>> }
        >>>
        >>> tree = subside(data)
        >>> # tree should be --> TreeValue({
        >>> #    'a': raw({'a': 1, 'k': '233', 'x': {'c': 3, 'd': [5, 7]}}),
        >>> #    'b': raw({'a': 2, 'k': '233', 'x': {'c': 4, 'd': [6, 8]}}),
        >>> #}), all structures above the tree values are subsided to the bottom of the tree.
    """

    def _build_func(v) -> Tuple[int, iter, Callable]:
        if dict_ and isinstance(v, dict):
            results = sorted([(key, _build_func(value_)) for key, value_ in v.items()])

            def _new_func(*args):
                position = 0
                returns = {}
                for key, (cnt, _, func) in results:
                    returns[key] = func(*args[position:position + cnt])
                    position += cnt

                return type(v)(returns)

            return sum([cnt for _, (cnt, _, _) in results]), chain(
                *[it for _, (_, it, _) in results]), _new_func
        elif (list_ and isinstance(v, list)) or (tuple_ and isinstance(v, tuple)):
            results = [_build_func(item) for item in v]

            def _new_func(*args):
                position = 0
                returns = []
                for cnt, _, func in results:
                    returns.append(func(*args[position: position + cnt]))
                    position += cnt

                return type(v)(returns)

            return sum([cnt for cnt, _, _ in results]), chain(
                *[it for _, it, _ in results]), _new_func

        else:
            return 1, (v,).__iter__(), lambda x: x

    def _get_default_type(args):
        types = {type(item) for item in args if isinstance(item, TreeValue)}
        try:
            return common_direct_base(*list(types))
        except TypeError:
            return None

    count, iter_, builder = _build_func(value)
    arguments = list(iter_)
    return_type = return_type or _get_default_type(arguments) or TreeValue
    assert count == len(arguments)

    from ..func import func_treelize
    return func_treelize(return_type=return_type, **kwargs)(builder)(*arguments)


NO_RISE_TEMPLATE = SingletonMark("no_rise_template")


def rise(tree: _TreeValue, dict_: bool = True, list_: bool = True, tuple_: bool = True,
         template=NO_RISE_TEMPLATE):
    """
    Overview:
        Make the structure (dict, list, tuple) in value rise up to the top, above the tree value.

    Arguments:
        - tree (:obj:`_TreeValue`): Tree value object
        - `dict_` (:obj:`bool`): Enable dict rise, default is `True`.
        - `list_` (:obj:`bool`): Enable list rise, default is `True`.
        - `tuple_` (:obj:`bool`): Enable list rise, default is `True`.
        - template (:obj:): Rising template, default is `NO_RISE_TEMPLATE`, which means auto detect.

    Returns:
        - risen (:obj:): Risen value.

    Example:
        >>> t = TreeValue({'x': raw({'a': [1, 2], 'b': [2, 3]}), 'y': raw({'a': [5, 6, 7], 'b': [7, 8]})})
        >>> dt = rise(t)
        >>> # dt will be {'a': <TreeValue 1>, 'b': [<TreeValue 2>, <TreeValue 3>]}
        >>> # TreeValue 1 will be TreeValue({'x': [1, 2], 'y': [5, 6, 7]})
        >>> # TreeValue 2 will be TreeValue({'x': 2, 'y': 7})
        >>> # TreeValue 3 will be TreeValue({'x': 3, 'y': 8})
        >>>
        >>> t2 = TreeValue({'x': raw({'a': [1, 2], 'b': [2, 3]}), 'y': raw({'a': [5, 6], 'b': [7, 8]})})
        >>> dt2 = rise(t2)
        >>> # dt2 will be {'a': [<TreeValue 1>, <TreeValue 2>], 'b': [<TreeValue 3>, <TreeValue 4>]}
        >>> # TreeValue 1 will be TreeValue({'x': 1, 'y': 5})
        >>> # TreeValue 2 will be TreeValue({'x': 2, 'y': 6})
        >>> # TreeValue 3 will be TreeValue({'x': 2, 'y': 7})
        >>> # TreeValue 4 will be TreeValue({'x': 3, 'y': 8})
        >>>
        >>> dt3 = rise(t2, template={'a': None, 'b': None})
        >>> # dt3 will be {'a': <TreeValue 1>, 'b': <TreeValue 2>}
        >>> # TreeValue 1 will be TreeValue({'x': [1, 2], 'y': [5, 6]})
        >>> # TreeValue 2 will be TreeValue({'x': [2, 3], 'y': [7, 8]})
    """

    def _get_tree_builder(t: _TreeValue):
        if isinstance(t, TreeValue):
            results = sorted([(key, _get_tree_builder(value)) for key, value in t])

            def _new_func(*args):
                position = 0
                returns = {}
                for key, (cnt, _, func_) in results:
                    returns[key] = func_(*args[position:position + cnt])
                    position += cnt

                return tree.__class__(returns)

            return sum([cnt for _, (cnt, _, _) in results]), chain(
                *[iter_ for _, (_, iter_, _) in results]), _new_func

        else:
            return 1, (t,).__iter__(), lambda x: raw(x)

    def _get_common_structure_getter(*args, tpl=NO_RISE_TEMPLATE):
        if not args:
            return 0, [], lambda: clone(tree)

        types = [type(item) for item in args]
        if tpl is not NO_RISE_TEMPLATE:
            types.append(type(tpl))
        base_class = common_direct_base(*types)
        if dict_ and issubclass(base_class, dict):
            keysets = [tuple(sorted(item.keys())) for item in args]
            if len(set(keysets)) == 1:
                keyset = sorted(list(keysets)[0])
                results = [(key, _get_common_structure_getter(
                    *[item[key] for item in args],
                    tpl=NO_RISE_TEMPLATE if tpl is NO_RISE_TEMPLATE else tpl[key]
                )) for key in keyset]
                getter_lists = {key: getters for key, (_, getters, _) in results}

                def _new_func(*args_):
                    position = 0
                    result = {}
                    for key, (cnt, _, func) in results:
                        result[key] = func(*args_[position:position + cnt])
                        position += cnt

                    return base_class(result)

                return sum([cnt for _, (cnt, _, _) in results]), [
                    partial(lambda k, g, x: g(x[k]), key, getter)
                    for key in keyset for getter in getter_lists[key]], _new_func

        elif (list_ and issubclass(base_class, list)) \
                or (tuple_ and issubclass(base_class, tuple)):
            lengths = [len(item) for item in args]
            if len(set(lengths)) == 1:
                length = list(lengths)[0]
                results = [_get_common_structure_getter(
                    *[item[i] for item in args],
                    tpl=NO_RISE_TEMPLATE if tpl is NO_RISE_TEMPLATE else tpl[i]
                ) for i in range(length)]
                getter_lists = [getters for _, getters, _ in results]

                def _new_func(*args_):
                    position = 0
                    result = []
                    for cnt, _, func in results:
                        result.append(func(*args_[position: position + cnt]))
                        position += cnt

                    return base_class(result)

                return sum([cnt for cnt, _, _ in results]), [
                    partial(lambda i, g, x: g(x[i]), index, getter)
                    for index in range(length) for getter in getter_lists[index]], _new_func

        if tpl is not NO_RISE_TEMPLATE and ((dict_ and isinstance(tpl, dict)) or (
                list_ and isinstance(tpl, list)) or (tuple_ and isinstance(tpl, tuple))):
            raise ValueError("Template not able to be applied "
                             "for schema {tpl} expected but not match is some values.".format(tpl=repr(tpl)))

        return 1, [lambda x: x], (lambda x: x)

    value_count, value_iter, tree_builder = _get_tree_builder(tree)
    value_list = list(value_iter)
    assert value_count == len(value_list)

    meta_value_count, meta_value_getters, value_builder = _get_common_structure_getter(*value_list, tpl=template)
    assert meta_value_count == len(meta_value_getters)

    return value_builder(*[tree_builder(*[getter_(item_) for item_ in value_list]) for getter_ in meta_value_getters])


def reduce_(tree: _TreeValue, func):
    """
    Overview
        Reduce the tree to value.

    Arguments:
        - tree (:obj:`_TreeValue`): Tree value object
        - func (:obj:): Function for reducing

    Returns:
        - result (:obj:): Reduce result

    Examples:
        >>> from functools import reduce
        >>>
        >>> t = TreeValue({'a': 1, 'b': 2, 'x': {'c': 3, 'd': 4}})
        >>> reduce_(t, lambda **kwargs: sum(kwargs.values()))  # 10, 1 + 2 + (3 + 4)
        >>> reduce_(t, lambda **kwargs: reduce(lambda x, y: x * y, list(kwargs.values())))  # 24, 1 * 2 * (3 * 4)
    """

    def _recursion(t: _TreeValue):
        if isinstance(t, tree.__class__):
            _result = func(**{key: _recursion(value) for key, value in t})
            if isinstance(_result, TreeValue):
                return typetrans(_result, tree.__class__)
            else:
                return _result
        else:
            return t

    return _recursion(tree)
